package solutions.bitbadger.documents.core.tests.java;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.Field;
import solutions.bitbadger.documents.query.CountQuery;
import solutions.bitbadger.documents.core.tests.ForceDialect;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static solutions.bitbadger.documents.core.tests.TypesKt.TEST_TABLE;

/**
 * Unit tests for the `Count` object
 */
@DisplayName("Core | Java | Query | CountQuery")
final public class CountQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    public void cleanUp() {
        ForceDialect.none();
    }

    @Test
    @DisplayName("all generates correctly")
    public void all() {
        assertEquals(String.format("SELECT COUNT(*) AS it FROM %s", TEST_TABLE), CountQuery.all(TEST_TABLE),
                "Count query not constructed correctly");
    }

    @Test
    @DisplayName("byFields generates correctly | PostgreSQL")
    public void byFieldsPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("SELECT COUNT(*) AS it FROM %s WHERE data->>'test' = :field0", TEST_TABLE),
                CountQuery.byFields(TEST_TABLE, List.of(Field.equal("test", "", ":field0"))),
                "Count query not constructed correctly");
    }

    @Test
    @DisplayName("byFields generates correctly | SQLite")
    public void byFieldsSQLite() throws DocumentException {
        ForceDialect.sqlite();
        assertEquals(String.format("SELECT COUNT(*) AS it FROM %s WHERE data->>'test' = :field0", TEST_TABLE),
                CountQuery.byFields(TEST_TABLE, List.of(Field.equal("test", "", ":field0"))),
                "Count query not constructed correctly");
    }

    @Test
    @DisplayName("byContains generates correctly | PostgreSQL")
    public void byContainsPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("SELECT COUNT(*) AS it FROM %s WHERE data @> :criteria", TEST_TABLE),
                CountQuery.byContains(TEST_TABLE), "Count query not constructed correctly");
    }

    @Test
    @DisplayName("byContains fails | SQLite")
    public void byContainsSQLite() {
        ForceDialect.sqlite();
        assertThrows(DocumentException.class, () -> CountQuery.byContains(TEST_TABLE));
    }

    @Test
    @DisplayName("byJsonPath generates correctly | PostgreSQL")
    public void byJsonPathPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(
                String.format("SELECT COUNT(*) AS it FROM %s WHERE jsonb_path_exists(data, :path::jsonpath)",
                        TEST_TABLE),
                CountQuery.byJsonPath(TEST_TABLE), "Count query not constructed correctly");
    }

    @Test
    @DisplayName("byJsonPath fails | SQLite")
    public void byJsonPathSQLite() {
        ForceDialect.sqlite();
        assertThrows(DocumentException.class, () -> CountQuery.byJsonPath(TEST_TABLE));
    }
}
