package solutions.bitbadger.documents.core.tests.java;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.Field;
import solutions.bitbadger.documents.query.ExistsQuery;
import solutions.bitbadger.documents.core.tests.ForceDialect;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static solutions.bitbadger.documents.core.tests.TypesKt.TEST_TABLE;

/**
 * Unit tests for the `Exists` object
 */
@DisplayName("Core | Java | Query | ExistsQuery")
final public class ExistsQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    public void cleanUp() {
        ForceDialect.none();
    }

    @Test
    @DisplayName("byId generates correctly | PostgreSQL")
    public void byIdPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("SELECT EXISTS (SELECT 1 FROM %s WHERE data->>'id' = :id) AS it", TEST_TABLE),
                ExistsQuery.byId(TEST_TABLE), "Exists query not constructed correctly");
    }

    @Test
    @DisplayName("byId generates correctly | SQLite")
    public void byIdSQLite() throws DocumentException {
        ForceDialect.sqlite();
        assertEquals(String.format("SELECT EXISTS (SELECT 1 FROM %s WHERE data->>'id' = :id) AS it", TEST_TABLE),
                ExistsQuery.byId(TEST_TABLE), "Exists query not constructed correctly");
    }

    @Test
    @DisplayName("byFields generates correctly | PostgreSQL")
    public void byFieldsPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format(
                        "SELECT EXISTS (SELECT 1 FROM %s WHERE (data->>'it')::numeric = :test) AS it", TEST_TABLE),
                ExistsQuery.byFields(TEST_TABLE, List.of(Field.equal("it", 7, ":test"))),
                "Exists query not constructed correctly");
    }

    @Test
    @DisplayName("byFields generates correctly | SQLite")
    public void byFieldsSQLite() throws DocumentException {
        ForceDialect.sqlite();
        assertEquals(String.format("SELECT EXISTS (SELECT 1 FROM %s WHERE data->>'it' = :test) AS it", TEST_TABLE),
                ExistsQuery.byFields(TEST_TABLE, List.of(Field.equal("it", 7, ":test"))),
                "Exists query not constructed correctly");
    }

    @Test
    @DisplayName("byContains generates correctly | PostgreSQL")
    public void byContainsPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("SELECT EXISTS (SELECT 1 FROM %s WHERE data @> :criteria) AS it", TEST_TABLE),
                ExistsQuery.byContains(TEST_TABLE), "Exists query not constructed correctly");
    }

    @Test
    @DisplayName("byContains fails | SQLite")
    public void byContainsSQLite() {
        ForceDialect.sqlite();
        assertThrows(DocumentException.class, () -> ExistsQuery.byContains(TEST_TABLE));
    }

    @Test
    @DisplayName("byJsonPath generates correctly | PostgreSQL")
    public void byJsonPathPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format(
                        "SELECT EXISTS (SELECT 1 FROM %s WHERE jsonb_path_exists(data, :path::jsonpath)) AS it",
                        TEST_TABLE),
                ExistsQuery.byJsonPath(TEST_TABLE), "Exists query not constructed correctly");
    }

    @Test
    @DisplayName("byJsonPath fails | SQLite")
    public void byJsonPathSQLite() {
        ForceDialect.sqlite();
        assertThrows(DocumentException.class, () -> ExistsQuery.byJsonPath(TEST_TABLE));
    }
}
