package solutions.bitbadger.documents.core.tests.java.integration;

import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.Field;
import solutions.bitbadger.documents.core.tests.integration.ThrowawayDatabase;

import java.util.List;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static solutions.bitbadger.documents.core.tests.TypesKt.TEST_TABLE;
import static solutions.bitbadger.documents.java.extensions.ConnExt.*;

final public class ExistsFunctions {

    public static void byIdMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertTrue(existsById(db.getConn(), TEST_TABLE, "three"), "The document with ID \"three\" should exist");
    }

    public static void byIdNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertFalse(existsById(db.getConn(), TEST_TABLE, "seven"), "The document with ID \"seven\" should not exist");
    }

    public static void byFieldsMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertTrue(existsByFields(db.getConn(), TEST_TABLE, List.of(Field.equal("numValue", 10))),
                "Matching documents should have been found");
    }

    public static void byFieldsNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertFalse(existsByFields(db.getConn(), TEST_TABLE, List.of(Field.equal("nothing", "none"))),
                "No matching documents should have been found");
    }

    public static void byContainsMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertTrue(existsByContains(db.getConn(), TEST_TABLE, Map.of("value", "purple")),
                "Matching documents should have been found");
    }

    public static void byContainsNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertFalse(existsByContains(db.getConn(), TEST_TABLE, Map.of("value", "violet")),
                "Matching documents should not have been found");
    }

    public static void byJsonPathMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertTrue(existsByJsonPath(db.getConn(), TEST_TABLE, "$.numValue ? (@ == 10)"),
                "Matching documents should have been found");
    }

    public static void byJsonPathNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertFalse(existsByJsonPath(db.getConn(), TEST_TABLE, "$.numValue ? (@ == 10.1)"),
                "Matching documents should not have been found");
    }
}
