package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.PgDB;

/**
 * PostgreSQL integration tests for the `Find` object / `find*` connection extension functions
 */
@DisplayName("Core | Java | PostgreSQL: Find")
final public class PostgreSQLFindIT {

    @Test
    @DisplayName("all retrieves all documents")
    public void allDefault() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.allDefault(db);
        }
    }

    @Test
    @DisplayName("all sorts data ascending")
    public void allAscending() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.allAscending(db);
        }
    }

    @Test
    @DisplayName("all sorts data descending")
    public void allDescending() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.allDescending(db);
        }
    }

    @Test
    @DisplayName("all sorts data numerically")
    public void allNumOrder() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.allNumOrder(db);
        }
    }

    @Test
    @DisplayName("all succeeds with an empty table")
    public void allEmpty() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.allEmpty(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    public void byIdString() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byIdString(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    public void byIdNumber() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byIdNumber(db);
        }
    }

    @Test
    @DisplayName("byId returns null when a matching ID is not found")
    public void byIdNotFound() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byIdNotFound(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents")
    public void byFieldsMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byFieldsMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    public void byFieldsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    public void byFieldsMatchNumIn() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byFieldsMatchNumIn(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match")
    public void byFieldsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    public void byFieldsMatchInArray() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byFieldsMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    public void byFieldsNoMatchInArray() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byFieldsNoMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byContains retrieves matching documents")
    public void byContainsMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byContainsMatch(db);
        }
    }

    @Test
    @DisplayName("byContains retrieves ordered matching documents")
    public void byContainsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byContainsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byContains succeeds when no documents match")
    public void byContainsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byContainsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byJsonPath retrieves matching documents")
    public void byJsonPathMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byJsonPathMatch(db);
        }
    }

    @Test
    @DisplayName("byJsonPath retrieves ordered matching documents")
    public void byJsonPathMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byJsonPathMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byJsonPath succeeds when no documents match")
    public void byJsonPathNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.byJsonPathNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    public void firstByFieldsMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByFieldsMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    public void firstByFieldsMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByFieldsMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    public void firstByFieldsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByFields returns null when no document matches")
    public void firstByFieldsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByContains retrieves a matching document")
    public void firstByContainsMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByContainsMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByContains retrieves a matching document among many")
    public void firstByContainsMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByContainsMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByContains retrieves a matching document among many (ordered)")
    public void firstByContainsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByContainsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByContains returns null when no document matches")
    public void firstByContainsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByContainsNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document")
    public void firstByJsonPathMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByJsonPathMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many")
    public void firstByJsonPathMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByJsonPathMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many (ordered)")
    public void firstByJsonPathMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByJsonPathMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath returns null when no document matches")
    public void firstByJsonPathNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            FindFunctions.firstByJsonPathNoMatch(db);
        }
    }
}
