package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.PgDB;

/**
 * PostgreSQL integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName("Core | Java | PostgreSQL: Json")
final public class PostgreSQLJsonIT {

    @Test
    @DisplayName("all retrieves all documents")
    public void allDefault() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.allDefault(db);
        }
    }

    @Test
    @DisplayName("all succeeds with an empty table")
    public void allEmpty() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.allEmpty(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    public void byIdString() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byIdString(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    public void byIdNumber() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byIdNumber(db);
        }
    }

    @Test
    @DisplayName("byId returns null when a matching ID is not found")
    public void byIdNotFound() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byIdNotFound(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents")
    public void byFieldsMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    public void byFieldsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    public void byFieldsMatchNumIn() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsMatchNumIn(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match")
    public void byFieldsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    public void byFieldsMatchInArray() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    public void byFieldsNoMatchInArray() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsNoMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byContains retrieves matching documents")
    public void byContainsMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byContainsMatch(db);
        }
    }

    @Test
    @DisplayName("byContains retrieves ordered matching documents")
    public void byContainsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byContainsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byContains succeeds when no documents match")
    public void byContainsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byContainsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byJsonPath retrieves matching documents")
    public void byJsonPathMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byJsonPathMatch(db);
        }
    }

    @Test
    @DisplayName("byJsonPath retrieves ordered matching documents")
    public void byJsonPathMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byJsonPathMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byJsonPath succeeds when no documents match")
    public void byJsonPathNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byJsonPathNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    public void firstByFieldsMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByFieldsMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    public void firstByFieldsMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByFieldsMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    public void firstByFieldsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByFields returns null when no document matches")
    public void firstByFieldsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByContains retrieves a matching document")
    public void firstByContainsMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByContainsMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByContains retrieves a matching document among many")
    public void firstByContainsMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByContainsMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByContains retrieves a matching document among many (ordered)")
    public void firstByContainsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByContainsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByContains returns null when no document matches")
    public void firstByContainsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByContainsNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document")
    public void firstByJsonPathMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByJsonPathMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many")
    public void firstByJsonPathMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByJsonPathMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many (ordered)")
    public void firstByJsonPathMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByJsonPathMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath returns null when no document matches")
    public void firstByJsonPathNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByJsonPathNoMatch(db);
        }
    }

    @Test
    @DisplayName("writeAll retrieves all documents")
    public void writeAllDefault() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeAllDefault(db);
        }
    }

    @Test
    @DisplayName("writeAll succeeds with an empty table")
    public void writeAllEmpty() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeAllEmpty(db);
        }
    }

    @Test
    @DisplayName("writeById retrieves a document via a string ID")
    public void writeByIdString() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByIdString(db);
        }
    }

    @Test
    @DisplayName("writeById retrieves a document via a numeric ID")
    public void writeByIdNumber() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByIdNumber(db);
        }
    }

    @Test
    @DisplayName("writeById returns null when a matching ID is not found")
    public void writeByIdNotFound() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByIdNotFound(db);
        }
    }

    @Test
    @DisplayName("writeByFields retrieves matching documents")
    public void writeByFieldsMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByFieldsMatch(db);
        }
    }

    @Test
    @DisplayName("writeByFields retrieves ordered matching documents")
    public void writeByFieldsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("writeByFields retrieves matching documents with a numeric IN clause")
    public void writeByFieldsMatchNumIn() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByFieldsMatchNumIn(db);
        }
    }

    @Test
    @DisplayName("writeByFields succeeds when no documents match")
    public void writeByFieldsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("writeByFields retrieves matching documents with an IN_ARRAY comparison")
    public void writeByFieldsMatchInArray() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByFieldsMatchInArray(db);
        }
    }

    @Test
    @DisplayName("writeByFields succeeds when no documents match an IN_ARRAY comparison")
    public void writeByFieldsNoMatchInArray() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByFieldsNoMatchInArray(db);
        }
    }

    @Test
    @DisplayName("writeByContains retrieves matching documents")
    public void writeByContainsMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByContainsMatch(db);
        }
    }

    @Test
    @DisplayName("writeByContains retrieves ordered matching documents")
    public void writeByContainsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByContainsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("writeByContains succeeds when no documents match")
    public void writeByContainsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByContainsNoMatch(db);
        }
    }

    @Test
    @DisplayName("writeByJsonPath retrieves matching documents")
    public void writeByJsonPathMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByJsonPathMatch(db);
        }
    }

    @Test
    @DisplayName("writeByJsonPath retrieves ordered matching documents")
    public void writeByJsonPathMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByJsonPathMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("writeByJsonPath succeeds when no documents match")
    public void writeByJsonPathNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeByJsonPathNoMatch(db);
        }
    }

    @Test
    @DisplayName("writeFirstByFields retrieves a matching document")
    public void writeFirstByFieldsMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByFieldsMatchOne(db);
        }
    }

    @Test
    @DisplayName("writeFirstByFields retrieves a matching document among many")
    public void writeFirstByFieldsMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByFieldsMatchMany(db);
        }
    }

    @Test
    @DisplayName("writeFirstByFields retrieves a matching document among many (ordered)")
    public void writeFirstByFieldsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("writeFirstByFields returns null when no document matches")
    public void writeFirstByFieldsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("writeFirstByContains retrieves a matching document")
    public void writeFirstByContainsMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByContainsMatchOne(db);
        }
    }

    @Test
    @DisplayName("writeFirstByContains retrieves a matching document among many")
    public void writeFirstByContainsMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByContainsMatchMany(db);
        }
    }

    @Test
    @DisplayName("writeFirstByContains retrieves a matching document among many (ordered)")
    public void writeFirstByContainsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByContainsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("writeFirstByContains returns null when no document matches")
    public void writeFirstByContainsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByContainsNoMatch(db);
        }
    }

    @Test
    @DisplayName("writeFirstByJsonPath retrieves a matching document")
    public void writeFirstByJsonPathMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByJsonPathMatchOne(db);
        }
    }

    @Test
    @DisplayName("writeFirstByJsonPath retrieves a matching document among many")
    public void writeFirstByJsonPathMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByJsonPathMatchMany(db);
        }
    }

    @Test
    @DisplayName("writeFirstByJsonPath retrieves a matching document among many (ordered)")
    public void writeFirstByJsonPathMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByJsonPathMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("writeFirstByJsonPath returns null when no document matches")
    public void writeFirstByJsonPathNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.writeFirstByJsonPathNoMatch(db);
        }
    }
}
