package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.SQLiteDB;

import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * SQLite integration tests for the `Exists` object / `existsBy*` connection extension functions
 */
@DisplayName("Core | Java | SQLite: Exists")
final public class SQLiteExistsIT {

    @Test
    @DisplayName("byId returns true when a document matches the ID")
    public void byIdMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            ExistsFunctions.byIdMatch(db);
        }
    }

    @Test
    @DisplayName("byId returns false when no document matches the ID")
    public void byIdNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            ExistsFunctions.byIdNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields returns true when documents match")
    public void byFieldsMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            ExistsFunctions.byFieldsMatch(db);
        }
    }

    @Test
    @DisplayName("byFields returns false when no documents match")
    public void byFieldsNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            ExistsFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byContains fails")
    public void byContainsMatch() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> ExistsFunctions.byContainsMatch(db));
        }
    }

    @Test
    @DisplayName("byJsonPath fails")
    public void byJsonPathMatch() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> ExistsFunctions.byJsonPathMatch(db));
        }
    }
}
