package solutions.bitbadger.documents.core.tests

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.query.DeleteQuery
import kotlin.test.assertEquals

/**
 * Unit tests for the `Delete` object
 */
@DisplayName("Core | Kotlin | Query | DeleteQuery")
class DeleteQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    fun cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName("byId generates correctly | PostgreSQL")
    fun byIdPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "DELETE FROM $TEST_TABLE WHERE data->>'id' = :id",
            DeleteQuery.byId<String>(TEST_TABLE), "Delete query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byId generates correctly | SQLite")
    fun byIdSQLite() {
        ForceDialect.sqlite()
        assertEquals(
            "DELETE FROM $TEST_TABLE WHERE data->>'id' = :id",
            DeleteQuery.byId<String>(TEST_TABLE), "Delete query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byFields generates correctly | PostgreSQL")
    fun byFieldsPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "DELETE FROM $TEST_TABLE WHERE data->>'a' = :b",
            DeleteQuery.byFields(TEST_TABLE, listOf(Field.equal("a", "", ":b"))),
            "Delete query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byFields generates correctly | SQLite")
    fun byFieldsSQLite() {
        ForceDialect.sqlite()
        assertEquals(
            "DELETE FROM $TEST_TABLE WHERE data->>'a' = :b",
            DeleteQuery.byFields(TEST_TABLE, listOf(Field.equal("a", "", ":b"))),
            "Delete query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains generates correctly | PostgreSQL")
    fun byContainsPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "DELETE FROM $TEST_TABLE WHERE data @> :criteria",
            DeleteQuery.byContains(TEST_TABLE),
            "Delete query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains fails | SQLite")
    fun byContainsSQLite() {
        ForceDialect.sqlite()
        assertThrows<DocumentException> { DeleteQuery.byContains(TEST_TABLE) }
    }

    @Test
    @DisplayName("byJsonPath generates correctly | PostgreSQL")
    fun byJsonPathPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "DELETE FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)",
            DeleteQuery.byJsonPath(TEST_TABLE),
            "Delete query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byJsonPath fails | SQLite")
    fun byJsonPathSQLite() {
        ForceDialect.sqlite()
        assertThrows<DocumentException> { DeleteQuery.byJsonPath(TEST_TABLE) }
    }
}
