package solutions.bitbadger.documents.core.tests

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.query.ExistsQuery
import kotlin.test.assertEquals

/**
 * Unit tests for the `Exists` object
 */
@DisplayName("Core | Kotlin | Query | ExistsQuery")
class ExistsQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    fun cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName("byId generates correctly | PostgreSQL")
    fun byIdPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data->>'id' = :id) AS it",
            ExistsQuery.byId<String>(TEST_TABLE), "Exists query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byId generates correctly | SQLite")
    fun byIdSQLite() {
        ForceDialect.sqlite()
        assertEquals(
            "SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data->>'id' = :id) AS it",
            ExistsQuery.byId<String>(TEST_TABLE), "Exists query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byFields generates correctly | PostgreSQL")
    fun byFieldsPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE (data->>'it')::numeric = :test) AS it",
            ExistsQuery.byFields(TEST_TABLE, listOf(Field.equal("it", 7, ":test"))),
            "Exists query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byFields generates correctly | SQLite")
    fun byFieldsSQLite() {
        ForceDialect.sqlite()
        assertEquals(
            "SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data->>'it' = :test) AS it",
            ExistsQuery.byFields(TEST_TABLE, listOf(Field.equal("it", 7, ":test"))),
            "Exists query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains generates correctly | PostgreSQL")
    fun byContainsPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data @> :criteria) AS it",
            ExistsQuery.byContains(TEST_TABLE),
            "Exists query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byContains fails | SQLite")
    fun byContainsSQLite() {
        ForceDialect.sqlite()
        assertThrows<DocumentException> { ExistsQuery.byContains(TEST_TABLE) }
    }

    @Test
    @DisplayName("byJsonPath generates correctly | PostgreSQL")
    fun byJsonPathPostgres() {
        ForceDialect.postgres()
        assertEquals(
            "SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)) AS it",
            ExistsQuery.byJsonPath(TEST_TABLE), "Exists query not constructed correctly"
        )
    }

    @Test
    @DisplayName("byJsonPath fails | SQLite")
    fun byJsonPathSQLite() {
        ForceDialect.sqlite()
        assertThrows<DocumentException> { ExistsQuery.byJsonPath(TEST_TABLE) }
    }
}
