package solutions.bitbadger.documents.core.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.DocumentException
import kotlin.test.Test

/**
 * SQLite integration tests for the `Delete` object / `deleteBy*` connection extension functions
 */
@DisplayName("Core | Kotlin | SQLite: Delete")
class SQLiteDeleteIT {

    @Test
    @DisplayName("byId deletes a matching ID")
    fun byIdMatch() =
        SQLiteDB().use(DeleteFunctions::byIdMatch)

    @Test
    @DisplayName("byId succeeds when no ID matches")
    fun byIdNoMatch() =
        SQLiteDB().use(DeleteFunctions::byIdNoMatch)

    @Test
    @DisplayName("byFields deletes matching documents")
    fun byFieldsMatch() =
        SQLiteDB().use(DeleteFunctions::byFieldsMatch)

    @Test
    @DisplayName("byFields succeeds when no documents match")
    fun byFieldsNoMatch() =
        SQLiteDB().use(DeleteFunctions::byFieldsNoMatch)

    @Test
    @DisplayName("byContains fails")
    fun byContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(DeleteFunctions::byContainsMatch) }
    }

    @Test
    @DisplayName("byJsonPath fails")
    fun byJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(DeleteFunctions::byJsonPathMatch) }
    }
}
