package solutions.bitbadger.documents.groovy.tests

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.Dialect
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.*

/**
 * Unit tests for the `Dialect` enum
 */
@DisplayName('Groovy | Dialect')
class DialectTest {

    @Test
    @DisplayName('deriveFromConnectionString derives PostgreSQL correctly')
    void derivesPostgres() {
        assertEquals(Dialect.POSTGRESQL, Dialect.deriveFromConnectionString('jdbc:postgresql:db'),
                'Dialect should have been PostgreSQL')
    }

    @Test
    @DisplayName('deriveFromConnectionString derives SQLite correctly')
    void derivesSQLite() {
        assertEquals(Dialect.SQLITE, Dialect.deriveFromConnectionString('jdbc:sqlite:memory'),
                'Dialect should have been SQLite')
    }

    @Test
    @DisplayName('deriveFromConnectionString fails when the connection string is unknown')
    void deriveFailsWhenUnknown() {
        try {
            Dialect.deriveFromConnectionString 'SQL Server'
            fail 'Dialect derivation should have failed'
        } catch (DocumentException ex) {
            assertNotNull ex.message, 'The exception message should not have been null'
            assertTrue(ex.message.contains('[SQL Server]'),
                    'The connection string should have been in the exception message')
        }
    }
}
