package solutions.bitbadger.documents.groovy.tests.integration

import solutions.bitbadger.documents.Field

import static org.junit.jupiter.api.Assertions.*
import static solutions.bitbadger.documents.groovy.tests.Types.TEST_TABLE

final class CountFunctions {

    static void all(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should have been 5 documents in the table'
    }

    static void byFieldsNumeric(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals(3L, db.conn.countByFields(TEST_TABLE, List.of(Field.between('numValue', 10, 20))),
                'There should have been 3 matching documents')
    }

    static void byFieldsAlpha(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals(1L, db.conn.countByFields(TEST_TABLE, List.of(Field.between('value', 'aardvark', 'apple'))),
                'There should have been 1 matching document')
    }

    static void byContainsMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals(2L, db.conn.countByContains(TEST_TABLE, Map.of('value', 'purple')),
                'There should have been 2 matching documents')
    }

    static void byContainsNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals(0L, db.conn.countByContains(TEST_TABLE, Map.of('value', 'magenta')),
                'There should have been no matching documents')
    }

    static void byJsonPathMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals(2L, db.conn.countByJsonPath(TEST_TABLE, '$.numValue ? (@ < 5)'),
                'There should have been 2 matching documents')
    }

    static void byJsonPathNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals(0L, db.conn.countByJsonPath(TEST_TABLE, '$.numValue ? (@ > 100)'),
                'There should have been no matching documents')
    }
}
