package solutions.bitbadger.documents.groovy.tests.integration

import solutions.bitbadger.documents.Field

import static org.junit.jupiter.api.Assertions.*
import static solutions.bitbadger.documents.groovy.tests.Types.TEST_TABLE

final class DeleteFunctions {

    static void byIdMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should be 5 documents in the table'
        db.conn.deleteById TEST_TABLE, 'four'
        assertEquals 4L, db.conn.countAll(TEST_TABLE), 'There should now be 4 documents in the table'
    }

    static void byIdNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should be 5 documents in the table'
        db.conn.deleteById TEST_TABLE, 'negative four'
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should still be 5 documents in the table'
    }

    static void byFieldsMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should be 5 documents in the table'
        db.conn.deleteByFields TEST_TABLE, List.of(Field.notEqual('value', 'purple'))
        assertEquals 2L, db.conn.countAll(TEST_TABLE), 'There should now be 2 documents in the table'
    }

    static void byFieldsNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should be 5 documents in the table'
        db.conn.deleteByFields TEST_TABLE, List.of(Field.equal('value', 'crimson'))
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should still be 5 documents in the table'
    }

    static void byContainsMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should be 5 documents in the table'
        db.conn.deleteByContains TEST_TABLE, Map.of('value', 'purple')
        assertEquals 3L, db.conn.countAll(TEST_TABLE), 'There should now be 3 documents in the table'
    }

    static void byContainsNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should be 5 documents in the table'
        db.conn.deleteByContains TEST_TABLE, Map.of('target', 'acquired')
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should still be 5 documents in the table'
    }

    static void byJsonPathMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should be 5 documents in the table'
        db.conn.deleteByJsonPath TEST_TABLE, '$.value ? (@ == "purple")'
        assertEquals 3L, db.conn.countAll(TEST_TABLE), 'There should now be 3 documents in the table'
    }

    static void byJsonPathNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should be 5 documents in the table'
        db.conn.deleteByJsonPath TEST_TABLE, '$.numValue ? (@ > 100)'
        assertEquals 5L, db.conn.countAll(TEST_TABLE), 'There should still be 5 documents in the table'
    }
}
