package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `Count` object / `count*` connection extension functions
 */
@DisplayName("Groovy | SQLite: Count")
final class SQLiteCountIT {

    @Test
    @DisplayName("all counts all documents")
    void all() {
        new SQLiteDB().withCloseable CountFunctions.&all
    }

    @Test
    @DisplayName("byFields counts documents by a numeric value")
    void byFieldsNumeric() {
        new SQLiteDB().withCloseable CountFunctions.&byFieldsNumeric
    }

    @Test
    @DisplayName("byFields counts documents by a alphanumeric value")
    void byFieldsAlpha() {
        new SQLiteDB().withCloseable CountFunctions.&byFieldsAlpha
    }

    @Test
    @DisplayName("byContains fails")
    void byContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { CountFunctions.byContainsMatch db }
        }
    }

    @Test
    @DisplayName("byJsonPath fails")
    void byJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { CountFunctions.byJsonPathMatch db }
        }
    }
}
