package solutions.bitbadger.documents.groovy.tests.integration

import solutions.bitbadger.documents.Configuration
import solutions.bitbadger.documents.Parameter
import solutions.bitbadger.documents.ParameterType
import solutions.bitbadger.documents.java.DocumentConfig
import solutions.bitbadger.documents.java.Results

import static solutions.bitbadger.documents.groovy.tests.Types.TEST_TABLE

/**
 * A wrapper for a throwaway SQLite database
 */
class SQLiteDB implements ThrowawayDatabase {

    SQLiteDB() {
        Configuration.setConnectionString "jdbc:sqlite:${dbName}.db"
        conn = Configuration.dbConn()
        conn.ensureTable TEST_TABLE

        // Use a Jackson-based document serializer for testing
        DocumentConfig.serializer = new JacksonDocumentSerializer()
    }

    void close() {
        conn.close()
        new File("${dbName}.db").delete()
        Configuration.setConnectionString null
    }

    boolean dbObjectExists(String name) {
        conn.customScalar("SELECT EXISTS (SELECT 1 FROM sqlite_master WHERE name = :name) AS it",
                List.of(new Parameter(":name", ParameterType.STRING, name)), Boolean, Results.&toExists)
    }
}
