package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `Delete` object / `deleteBy*` connection extension functions
 */
@DisplayName('Groovy | SQLite: Delete')
final class SQLiteDeleteIT {

    @Test
    @DisplayName('byId deletes a matching ID')
    void byIdMatch() {
        new SQLiteDB().withCloseable DeleteFunctions.&byIdMatch
    }

    @Test
    @DisplayName('byId succeeds when no ID matches')
    void byIdNoMatch() {
        new SQLiteDB().withCloseable DeleteFunctions.&byIdNoMatch
    }

    @Test
    @DisplayName('byFields deletes matching documents')
    void byFieldsMatch() {
        new SQLiteDB().withCloseable DeleteFunctions.&byFieldsMatch
    }

    @Test
    @DisplayName('byFields succeeds when no documents match')
    void byFieldsNoMatch() {
        new SQLiteDB().withCloseable DeleteFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byContains fails')
    void byContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { DeleteFunctions.byContainsMatch db }
        }
    }

    @Test
    @DisplayName('byJsonPath fails')
    void byJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { DeleteFunctions.byJsonPathMatch db }
        }
    }
}
