package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `Find` object / `find*` connection extension functions
 */
@DisplayName('Groovy | SQLite: Find')
final class SQLiteFindIT {

    @Test
    @DisplayName('all retrieves all documents')
    void allDefault() {
        new SQLiteDB().withCloseable FindFunctions.&allDefault
    }

    @Test
    @DisplayName('all sorts data ascending')
    void allAscending() {
        new SQLiteDB().withCloseable FindFunctions.&allAscending
    }

    @Test
    @DisplayName('all sorts data descending')
    void allDescending() {
        new SQLiteDB().withCloseable FindFunctions.&allDescending
    }

    @Test
    @DisplayName('all sorts data numerically')
    void allNumOrder() {
        new SQLiteDB().withCloseable FindFunctions.&allNumOrder
    }

    @Test
    @DisplayName('all succeeds with an empty table')
    void allEmpty() {
        new SQLiteDB().withCloseable FindFunctions.&allEmpty
    }

    @Test
    @DisplayName('byId retrieves a document via a string ID')
    void byIdString() {
        new SQLiteDB().withCloseable FindFunctions.&byIdString
    }

    @Test
    @DisplayName('byId retrieves a document via a numeric ID')
    void byIdNumber() {
        new SQLiteDB().withCloseable FindFunctions.&byIdNumber
    }

    @Test
    @DisplayName('byId returns null when a matching ID is not found')
    void byIdNotFound() {
        new SQLiteDB().withCloseable FindFunctions.&byIdNotFound
    }

    @Test
    @DisplayName('byFields retrieves matching documents')
    void byFieldsMatch() {
        new SQLiteDB().withCloseable FindFunctions.&byFieldsMatch
    }

    @Test
    @DisplayName('byFields retrieves ordered matching documents')
    void byFieldsMatchOrdered() {
        new SQLiteDB().withCloseable FindFunctions.&byFieldsMatchOrdered
    }

    @Test
    @DisplayName('byFields retrieves matching documents with a numeric IN clause')
    void byFieldsMatchNumIn() {
        new SQLiteDB().withCloseable FindFunctions.&byFieldsMatchNumIn
    }

    @Test
    @DisplayName('byFields succeeds when no documents match')
    void byFieldsNoMatch() {
        new SQLiteDB().withCloseable FindFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byFields retrieves matching documents with an IN_ARRAY comparison')
    void byFieldsMatchInArray() {
        new SQLiteDB().withCloseable FindFunctions.&byFieldsMatchInArray
    }

    @Test
    @DisplayName('byFields succeeds when no documents match an IN_ARRAY comparison')
    void byFieldsNoMatchInArray() {
        new SQLiteDB().withCloseable FindFunctions.&byFieldsNoMatchInArray
    }

    @Test
    @DisplayName('byContains fails')
    void byContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { FindFunctions.byContainsMatch db }
        }
    }

    @Test
    @DisplayName('byJsonPath fails')
    void byJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { FindFunctions.byJsonPathMatch db }
        }
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document')
    void firstByFieldsMatchOne() {
        new SQLiteDB().withCloseable FindFunctions.&firstByFieldsMatchOne
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many')
    void firstByFieldsMatchMany() {
        new SQLiteDB().withCloseable FindFunctions.&firstByFieldsMatchMany
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many (ordered)')
    void firstByFieldsMatchOrdered() {
        new SQLiteDB().withCloseable FindFunctions.&firstByFieldsMatchOrdered
    }

    @Test
    @DisplayName('firstByFields returns null when no document matches')
    void firstByFieldsNoMatch() {
        new SQLiteDB().withCloseable FindFunctions.&firstByFieldsNoMatch
    }

    @Test
    @DisplayName('firstByContains fails')
    void firstByContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { FindFunctions.firstByContainsMatchOne db }
        }
    }

    @Test
    @DisplayName('firstByJsonPath fails')
    void firstByJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { FindFunctions.firstByJsonPathMatchOne db }
        }
    }
}
