package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName('Groovy | SQLite: Json')
final class SQLiteJsonIT {

    @Test
    @DisplayName('all retrieves all documents')
    void allDefault() {
        new SQLiteDB().withCloseable JsonFunctions.&allDefault
    }

    @Test
    @DisplayName('all succeeds with an empty table')
    void allEmpty() {
        new SQLiteDB().withCloseable JsonFunctions.&allEmpty
    }

    @Test
    @DisplayName('byId retrieves a document via a string ID')
    void byIdString() {
        new SQLiteDB().withCloseable JsonFunctions.&byIdString
    }

    @Test
    @DisplayName('byId retrieves a document via a numeric ID')
    void byIdNumber() {
        new SQLiteDB().withCloseable JsonFunctions.&byIdNumber
    }

    @Test
    @DisplayName('byId returns null when a matching ID is not found')
    void byIdNotFound() {
        new SQLiteDB().withCloseable JsonFunctions.&byIdNotFound
    }

    @Test
    @DisplayName('byFields retrieves matching documents')
    void byFieldsMatch() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsMatch
    }

    @Test
    @DisplayName('byFields retrieves ordered matching documents')
    void byFieldsMatchOrdered() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsMatchOrdered
    }

    @Test
    @DisplayName('byFields retrieves matching documents with a numeric IN clause')
    void byFieldsMatchNumIn() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsMatchNumIn
    }

    @Test
    @DisplayName('byFields succeeds when no documents match')
    void byFieldsNoMatch() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byFields retrieves matching documents with an IN_ARRAY comparison')
    void byFieldsMatchInArray() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsMatchInArray
    }

    @Test
    @DisplayName('byFields succeeds when no documents match an IN_ARRAY comparison')
    void byFieldsNoMatchInArray() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsNoMatchInArray
    }

    @Test
    @DisplayName('byContains fails')
    void byContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.byContainsMatch db }
        }
    }

    @Test
    @DisplayName('byJsonPath fails')
    void byJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.byJsonPathMatch db }
        }
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document')
    void firstByFieldsMatchOne() {
        new SQLiteDB().withCloseable JsonFunctions.&firstByFieldsMatchOne
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many')
    void firstByFieldsMatchMany() {
        new SQLiteDB().withCloseable JsonFunctions.&firstByFieldsMatchMany
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many (ordered)')
    void firstByFieldsMatchOrdered() {
        new SQLiteDB().withCloseable JsonFunctions.&firstByFieldsMatchOrdered
    }

    @Test
    @DisplayName('firstByFields returns null when no document matches')
    void firstByFieldsNoMatch() {
        new SQLiteDB().withCloseable JsonFunctions.&firstByFieldsNoMatch
    }

    @Test
    @DisplayName('firstByContains fails')
    void firstByContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.firstByContainsMatchOne db }
        }
    }

    @Test
    @DisplayName('firstByJsonPath fails')
    void firstByJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.firstByJsonPathMatchOne db }
        }
    }

    @Test
    @DisplayName('writeAll retrieves all documents')
    void writeAllDefault() {
        new SQLiteDB().withCloseable JsonFunctions.&writeAllDefault
    }

    @Test
    @DisplayName('writeAll succeeds with an empty table')
    void writeAllEmpty() {
        new SQLiteDB().withCloseable JsonFunctions.&writeAllEmpty
    }

    @Test
    @DisplayName('writeById retrieves a document via a string ID')
    void writeByIdString() {
        new SQLiteDB().withCloseable JsonFunctions.&writeByIdString
    }

    @Test
    @DisplayName('writeById retrieves a document via a numeric ID')
    void writeByIdNumber() {
        new SQLiteDB().withCloseable JsonFunctions.&writeByIdNumber
    }

    @Test
    @DisplayName('writeById returns null when a matching ID is not found')
    void writeByIdNotFound() {
        new SQLiteDB().withCloseable JsonFunctions.&writeByIdNotFound
    }

    @Test
    @DisplayName('writeByFields retrieves matching documents')
    void writeByFieldsMatch() {
        new SQLiteDB().withCloseable JsonFunctions.&writeByFieldsMatch
    }

    @Test
    @DisplayName('writeByFields retrieves ordered matching documents')
    void writeByFieldsMatchOrdered() {
        new SQLiteDB().withCloseable JsonFunctions.&writeByFieldsMatchOrdered
    }

    @Test
    @DisplayName('writeByFields retrieves matching documents with a numeric IN clause')
    void writeByFieldsMatchNumIn() {
        new SQLiteDB().withCloseable JsonFunctions.&writeByFieldsMatchNumIn
    }

    @Test
    @DisplayName('writeByFields succeeds when no documents match')
    void writeByFieldsNoMatch() {
        new SQLiteDB().withCloseable JsonFunctions.&writeByFieldsNoMatch
    }

    @Test
    @DisplayName('writeByFields retrieves matching documents with an IN_ARRAY comparison')
    void writeByFieldsMatchInArray() {
        new SQLiteDB().withCloseable JsonFunctions.&writeByFieldsMatchInArray
    }

    @Test
    @DisplayName('writeByFields succeeds when no documents match an IN_ARRAY comparison')
    void writeByFieldsNoMatchInArray() {
        new SQLiteDB().withCloseable JsonFunctions.&writeByFieldsNoMatchInArray
    }

    @Test
    @DisplayName('writeByContains fails')
    void writeByContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.writeByContainsMatch db }
        }
    }

    @Test
    @DisplayName('writeByJsonPath fails')
    void writeByJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.writeByJsonPathMatch db }
        }
    }

    @Test
    @DisplayName('writeFirstByFields retrieves a matching document')
    void writeFirstByFieldsMatchOne() {
        new SQLiteDB().withCloseable JsonFunctions.&writeFirstByFieldsMatchOne
    }

    @Test
    @DisplayName('writeFirstByFields retrieves a matching document among many')
    void writeFirstByFieldsMatchMany() {
        new SQLiteDB().withCloseable JsonFunctions.&writeFirstByFieldsMatchMany
    }

    @Test
    @DisplayName('writeFirstByFields retrieves a matching document among many (ordered)')
    void writeFirstByFieldsMatchOrdered() {
        new SQLiteDB().withCloseable JsonFunctions.&writeFirstByFieldsMatchOrdered
    }

    @Test
    @DisplayName('writeFirstByFields returns null when no document matches')
    void writeFirstByFieldsNoMatch() {
        new SQLiteDB().withCloseable JsonFunctions.&writeFirstByFieldsNoMatch
    }

    @Test
    @DisplayName('writeFirstByContains fails')
    void writeFirstByContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.writeFirstByContainsMatchOne db }
        }
    }

    @Test
    @DisplayName('writeFirstByJsonPath fails')
    void writeFirstByJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.writeFirstByJsonPathMatchOne db }
        }
    }
}
