package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `RemoveFields` object / `removeFieldsBy*` connection extension functions
 */
@DisplayName('Groovy | SQLite: RemoveFields')
final class SQLiteRemoveFieldsIT {

    @Test
    @DisplayName('byId removes fields from an existing document')
    void byIdMatchFields() {
        new SQLiteDB().withCloseable RemoveFieldsFunctions.&byIdMatchFields
    }

    @Test
    @DisplayName('byId succeeds when fields do not exist on an existing document')
    void byIdMatchNoFields() {
        new SQLiteDB().withCloseable RemoveFieldsFunctions.&byIdMatchNoFields
    }

    @Test
    @DisplayName('byId succeeds when no document exists')
    void byIdNoMatch() {
        new SQLiteDB().withCloseable RemoveFieldsFunctions.&byIdNoMatch
    }

    @Test
    @DisplayName('byFields removes fields from matching documents')
    void byFieldsMatchFields() {
        new SQLiteDB().withCloseable RemoveFieldsFunctions.&byFieldsMatchFields
    }

    @Test
    @DisplayName('byFields succeeds when fields do not exist on matching documents')
    void byFieldsMatchNoFields() {
        new SQLiteDB().withCloseable RemoveFieldsFunctions.&byFieldsMatchNoFields
    }

    @Test
    @DisplayName('byFields succeeds when no matching documents exist')
    void byFieldsNoMatch() {
        new SQLiteDB().withCloseable RemoveFieldsFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byContains fails')
    void byContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { RemoveFieldsFunctions.byContainsMatchFields db }
        }
    }

    @Test
    @DisplayName('byJsonPath fails')
    void byJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { RemoveFieldsFunctions.byJsonPathMatchFields db }
        }
    }
}
