package solutions.bitbadger.documents.kotlinx.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.DocumentException
import kotlin.test.Test

/**
 * SQLite integration tests for the `Exists` object / `existsBy*` connection extension functions
 */
@DisplayName("KotlinX | SQLite: Exists")
class SQLiteExistsIT {

    @Test
    @DisplayName("byId returns true when a document matches the ID")
    fun byIdMatch() =
        SQLiteDB().use(ExistsFunctions::byIdMatch)

    @Test
    @DisplayName("byId returns false when no document matches the ID")
    fun byIdNoMatch() =
        SQLiteDB().use(ExistsFunctions::byIdNoMatch)

    @Test
    @DisplayName("byFields returns true when documents match")
    fun byFieldsMatch() =
        SQLiteDB().use(ExistsFunctions::byFieldsMatch)

    @Test
    @DisplayName("byFields returns false when no documents match")
    fun byFieldsNoMatch() =
        SQLiteDB().use(ExistsFunctions::byFieldsNoMatch)

    @Test
    @DisplayName("byContains fails")
    fun byContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(ExistsFunctions::byContainsMatch) }
    }

    @Test
    @DisplayName("byJsonPath fails")
    fun byJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(ExistsFunctions::byJsonPathMatch) }
    }
}
