package solutions.bitbadger.documents.scala.tests

import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.{AfterEach, DisplayName, Test}
import solutions.bitbadger.documents.{DocumentException, Field}
import solutions.bitbadger.documents.query.CountQuery

import scala.jdk.CollectionConverters.*

@DisplayName("Scala | Query | CountQuery")
class CountQueryTest:

  /**
   * Clear the connection string (resets Dialect)
   */
  @AfterEach
  def cleanUp(): Unit =
    ForceDialect.none()

  @Test
  @DisplayName("all generates correctly")
  def all(): Unit =
    assertEquals(s"SELECT COUNT(*) AS it FROM $TEST_TABLE", CountQuery.all(TEST_TABLE),
      "Count query not constructed correctly")

  @Test
  @DisplayName("byFields generates correctly | PostgreSQL")
  def byFieldsPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data->>'test' = :field0",
      CountQuery.byFields(TEST_TABLE, List(Field.equal("test", "", ":field0")).asJava),
      "Count query not constructed correctly")

  @Test
  @DisplayName("byFields generates correctly | SQLite")
  def byFieldsSQLite(): Unit =
    ForceDialect.sqlite()
    assertEquals(s"SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data->>'test' = :field0",
      CountQuery.byFields(TEST_TABLE, List(Field.equal("test", "", ":field0")).asJava),
      "Count query not constructed correctly")

  @Test
  @DisplayName("byContains generates correctly | PostgreSQL")
  def byContainsPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data @> :criteria", CountQuery.byContains(TEST_TABLE),
      "Count query not constructed correctly")

  @Test
  @DisplayName("byContains fails | SQLite")
  def byContainsSQLite(): Unit =
    ForceDialect.sqlite()
    assertThrows(classOf[DocumentException], () => CountQuery.byContains(TEST_TABLE))

  @Test
  @DisplayName("byJsonPath generates correctly | PostgreSQL")
  def byJsonPathPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)",
      CountQuery.byJsonPath(TEST_TABLE), "Count query not constructed correctly")

  @Test
  @DisplayName("byJsonPath fails | SQLite")
  def byJsonPathSQLite(): Unit =
    ForceDialect.sqlite()
    assertThrows(classOf[DocumentException], () => CountQuery.byJsonPath(TEST_TABLE))
