package solutions.bitbadger.documents.scala.tests

import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.{AfterEach, DisplayName, Test}
import solutions.bitbadger.documents.{DocumentException, Field}
import solutions.bitbadger.documents.query.DeleteQuery

import scala.jdk.CollectionConverters.*

@DisplayName("Scala | Query | DeleteQuery")
class DeleteQueryTest:

  /**
   * Clear the connection string (resets Dialect)
   */
  @AfterEach
  def cleanUp(): Unit =
    ForceDialect.none()

  @Test
  @DisplayName("byId generates correctly | PostgreSQL")
  def byIdPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"DELETE FROM $TEST_TABLE WHERE data->>'id' = :id", DeleteQuery.byId(TEST_TABLE),
      "Delete query not constructed correctly")

  @Test
  @DisplayName("byId generates correctly | SQLite")
  def byIdSQLite(): Unit =
    ForceDialect.sqlite()
    assertEquals(s"DELETE FROM $TEST_TABLE WHERE data->>'id' = :id", DeleteQuery.byId(TEST_TABLE),
      "Delete query not constructed correctly")

  @Test
  @DisplayName("byFields generates correctly | PostgreSQL")
  def byFieldsPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"DELETE FROM $TEST_TABLE WHERE data->>'a' = :b",
      DeleteQuery.byFields(TEST_TABLE, List(Field.equal("a", "", ":b")).asJava),
      "Delete query not constructed correctly")

  @Test
  @DisplayName("byFields generates correctly | SQLite")
  def byFieldsSQLite(): Unit =
    ForceDialect.sqlite()
    assertEquals(s"DELETE FROM $TEST_TABLE WHERE data->>'a' = :b",
      DeleteQuery.byFields(TEST_TABLE, List(Field.equal("a", "", ":b")).asJava),
      "Delete query not constructed correctly")

  @Test
  @DisplayName("byContains generates correctly | PostgreSQL")
  def byContainsPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"DELETE FROM $TEST_TABLE WHERE data @> :criteria", DeleteQuery.byContains(TEST_TABLE),
      "Delete query not constructed correctly")

  @Test
  @DisplayName("byContains fails | SQLite")
  def byContainsSQLite(): Unit =
    ForceDialect.sqlite()
    assertThrows(classOf[DocumentException], () => DeleteQuery.byContains(TEST_TABLE))

  @Test
  @DisplayName("byJsonPath generates correctly | PostgreSQL")
  def byJsonPathPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"DELETE FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)",
      DeleteQuery.byJsonPath(TEST_TABLE), "Delete query not constructed correctly")

  @Test
  @DisplayName("byJsonPath fails | SQLite")
  def byJsonPathSQLite(): Unit =
    ForceDialect.sqlite()
    assertThrows(classOf[DocumentException], () => DeleteQuery.byJsonPath(TEST_TABLE))
