package solutions.bitbadger.documents.scala.tests

import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.{DisplayName, Test}
import solutions.bitbadger.documents.{Dialect, DocumentException}

@DisplayName("Scala | Dialect")
class DialectTest:

  @Test
  @DisplayName("deriveFromConnectionString derives PostgreSQL correctly")
  def derivesPostgres(): Unit =
    assertEquals(Dialect.POSTGRESQL, Dialect.deriveFromConnectionString("jdbc:postgresql:db"),
      "Dialect should have been PostgreSQL")

  @Test
  @DisplayName("deriveFromConnectionString derives SQLite correctly")
  def derivesSQLite(): Unit =
    assertEquals(Dialect.SQLITE, Dialect.deriveFromConnectionString("jdbc:sqlite:memory"),
      "Dialect should have been SQLite")

  @Test
  @DisplayName("deriveFromConnectionString fails when the connection string is unknown")
  def deriveFailsWhenUnknown(): Unit =
    try
      Dialect.deriveFromConnectionString("SQL Server")
      fail("Dialect derivation should have failed")
    catch
      case ex: DocumentException =>
        assertNotNull(ex.getMessage, "The exception message should not have been null")
        assertTrue(ex.getMessage.contains("[SQL Server]"),
          "The connection string should have been in the exception message")
