package solutions.bitbadger.documents.scala.tests

import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.{AfterEach, DisplayName, Test}
import solutions.bitbadger.documents.{DocumentException, Field}
import solutions.bitbadger.documents.query.ExistsQuery

import scala.jdk.CollectionConverters.*

@DisplayName("Scala | Query | ExistsQuery")
class ExistsQueryTest:

  /**
   * Clear the connection string (resets Dialect)
   */
  @AfterEach
  def cleanUp(): Unit =
    ForceDialect.none()

  @Test
  @DisplayName("byId generates correctly | PostgreSQL")
  def byIdPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data->>'id' = :id) AS it",
      ExistsQuery.byId(TEST_TABLE), "Exists query not constructed correctly")

  @Test
  @DisplayName("byId generates correctly | SQLite")
  def byIdSQLite(): Unit =
    ForceDialect.sqlite()
    assertEquals(s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data->>'id' = :id) AS it",
      ExistsQuery.byId(TEST_TABLE), "Exists query not constructed correctly")

  @Test
  @DisplayName("byFields generates correctly | PostgreSQL")
  def byFieldsPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE (data->>'it')::numeric = :test) AS it",
      ExistsQuery.byFields(TEST_TABLE, List(Field.equal("it", 7, ":test")).asJava),
      "Exists query not constructed correctly")

  @Test
  @DisplayName("byFields generates correctly | SQLite")
  def byFieldsSQLite(): Unit =
    ForceDialect.sqlite()
    assertEquals(s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data->>'it' = :test) AS it",
      ExistsQuery.byFields(TEST_TABLE, List(Field.equal("it", 7, ":test")).asJava),
      "Exists query not constructed correctly")

  @Test
  @DisplayName("byContains generates correctly | PostgreSQL")
  def byContainsPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data @> :criteria) AS it",
      ExistsQuery.byContains(TEST_TABLE), "Exists query not constructed correctly")

  @Test
  @DisplayName("byContains fails | SQLite")
  def byContainsSQLite(): Unit =
    ForceDialect.sqlite()
    assertThrows(classOf[DocumentException], () => ExistsQuery.byContains(TEST_TABLE))

  @Test
  @DisplayName("byJsonPath generates correctly | PostgreSQL")
  def byJsonPathPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)) AS it",
      ExistsQuery.byJsonPath(TEST_TABLE), "Exists query not constructed correctly")

  @Test
  @DisplayName("byJsonPath fails | SQLite")
  def byJsonPathSQLite(): Unit =
    ForceDialect.sqlite()
    assertThrows(classOf[DocumentException], () => ExistsQuery.byJsonPath(TEST_TABLE))
