package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.{DisplayName, Test}
import org.junit.jupiter.api.Assertions.assertThrows
import solutions.bitbadger.documents.DocumentException

import scala.util.Using

/**
 * SQLite integration tests for the `Delete` object / `deleteBy*` connection extension functions
 */
@DisplayName("Scala | SQLite: Delete")
class SQLiteDeleteIT:

  @Test
  @DisplayName("byId deletes a matching ID")
  def byIdMatch(): Unit =
    Using(SQLiteDB()) { db => DeleteFunctions.byIdMatch(db) }

  @Test
  @DisplayName("byId succeeds when no ID matches")
  def byIdNoMatch(): Unit =
    Using(SQLiteDB()) { db => DeleteFunctions.byIdNoMatch(db) }

  @Test
  @DisplayName("byFields deletes matching documents")
  def byFieldsMatch(): Unit =
    Using(SQLiteDB()) { db => DeleteFunctions.byFieldsMatch(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match")
  def byFieldsNoMatch(): Unit =
    Using(SQLiteDB()) { db => DeleteFunctions.byFieldsNoMatch(db) }

  @Test
  @DisplayName("byContains fails")
  def byContainsFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => DeleteFunctions.byContainsMatch(db)) }
  
  @Test
  @DisplayName("byJsonPath fails")
  def byJsonPathFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => DeleteFunctions.byJsonPathMatch(db)) }
