package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.Assertions.assertThrows
import org.junit.jupiter.api.{DisplayName, Test}
import solutions.bitbadger.documents.DocumentException

import scala.util.Using

/**
 * SQLite integration tests for the `Find` object / `find*` connection extension functions
 */
@DisplayName("Scala | SQLite: Find")
class SQLiteFindIT:

  @Test
  @DisplayName("all retrieves all documents")
  def allDefault(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.allDefault(db) }

  @Test
  @DisplayName("all sorts data ascending")
  def allAscending(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.allAscending(db) }

  @Test
  @DisplayName("all sorts data descending")
  def allDescending(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.allDescending(db) }

  @Test
  @DisplayName("all sorts data numerically")
  def allNumOrder(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.allNumOrder(db) }

  @Test
  @DisplayName("all succeeds with an empty table")
  def allEmpty(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.allEmpty(db) }

  @Test
  @DisplayName("byId retrieves a document via a string ID")
  def byIdString(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.byIdString(db) }

  @Test
  @DisplayName("byId retrieves a document via a numeric ID")
  def byIdNumber(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.byIdNumber(db) }

  @Test
  @DisplayName("byId returns null when a matching ID is not found")
  def byIdNotFound(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.byIdNotFound(db) }

  @Test
  @DisplayName("byFields retrieves matching documents")
  def byFieldsMatch(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.byFieldsMatch(db) }

  @Test
  @DisplayName("byFields retrieves ordered matching documents")
  def byFieldsMatchOrdered(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.byFieldsMatchOrdered(db) }

  @Test
  @DisplayName("byFields retrieves matching documents with a numeric IN clause")
  def byFieldsMatchNumIn(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.byFieldsMatchNumIn(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match")
  def byFieldsNoMatch(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.byFieldsNoMatch(db) }

  @Test
  @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
  def byFieldsMatchInArray(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.byFieldsMatchInArray(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
  def byFieldsNoMatchInArray(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.byFieldsNoMatchInArray(db) }

  @Test
  @DisplayName("byContains fails")
  def byContainsFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => FindFunctions.byContainsMatch(db)) }

  @Test
  @DisplayName("byJsonPath fails")
  def byJsonPathFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => FindFunctions.byJsonPathMatch(db)) }

  @Test
  @DisplayName("firstByFields retrieves a matching document")
  def firstByFieldsMatchOne(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.firstByFieldsMatchOne(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document among many")
  def firstByFieldsMatchMany(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.firstByFieldsMatchMany(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document among many (ordered)")
  def firstByFieldsMatchOrdered(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.firstByFieldsMatchOrdered(db) }

  @Test
  @DisplayName("firstByFields returns null when no document matches")
  def firstByFieldsNoMatch(): Unit =
    Using(SQLiteDB()) { db => FindFunctions.firstByFieldsNoMatch(db) }

  @Test
  @DisplayName("firstByContains fails")
  def firstByContainsFails(): Unit =
    Using(SQLiteDB()) { db =>
      assertThrows(classOf[DocumentException], () => FindFunctions.firstByContainsMatchOne(db))
    }

  @Test
  @DisplayName("firstByJsonPath fails")
  def firstByJsonPathFails(): Unit =
    Using(SQLiteDB()) { db =>
      assertThrows(classOf[DocumentException], () => FindFunctions.firstByJsonPathMatchOne(db))
    }
